import numpy as np
import pandas as pd

class LineupsData:
    pairs = []
    vecCB = []
    vecVis = []
    fids = []
    allDataLength = 0
    contentMode = "cbvis"
    visLayer = "FC7"

    def __init__(self, seed, contentMode, visLayer):
        self.visLayer = visLayer
        self.contentMode = contentMode

        (self.pairs, self.vecCB, self.vecVis, self.fids) = self.load_lineups_data()
        np.random.seed(seed)
        self.dtSizeCB = self.vecCB.shape[1]
        self.dtSizeVis = self.vecVis.shape[1]

    def load_lineups_data(self):  # return pairs, CBvectors, VisVectors, listFIDs
        vv = pd.read_csv("sourceData/personsVectors" + self.visLayer + ".csv", sep=';', header=None)
        vec_vis = np.asarray(vv)[:, 1:]
        vv2 = pd.read_csv("sourceData/personsVectors" + self.visLayer + ".csv", sep=';', header=None, dtype="str")
        fids = [str(f) for f in np.asarray(vv2)[:, 0]]

        self.allDataLength = len(fids)
        cb = pd.read_csv("sourceData/personsCBVectors.csv", sep=';', header=None)
        vec_cb = np.asarray(cb)

        pgs = pd.read_csv("sourceData/allPairs.csv", sep=';', header=0, dtype="str")
        pairs = np.asarray(pgs)

        return (pairs, vec_cb, vec_vis, fids)

    def next_batch(self, length):
        # batchIDs = np.random.choice(self.pairs, length)
        batchIDs = self.pairs[np.random.choice(self.pairs.shape[0], length, replace=False), :]

        batch_x11 = [self.vecCB[self.fids.index(a), :] for (a, b, c) in batchIDs]
        batch_x12 = [self.vecVis[self.fids.index(a), :] for (a, b, c) in batchIDs]

        batch_x21 = [self.vecCB[self.fids.index(b), :] for (a, b, c) in batchIDs]
        batch_x22 = [self.vecVis[self.fids.index(b), :] for (a, b, c) in batchIDs]

        batch_y = [c for (a, b, c) in batchIDs]
        return (batch_x11, batch_x12, batch_x21, batch_x22, batch_y)

    def test(self, length):
        # batchIDs = np.random.choice(self.pairs, length)
        fotoIDs = range(length)

        test_x1 = [self.vecCB[a, :] for a in fotoIDs]
        test_x2 = [self.vecVis[a, :] for a in fotoIDs]
        test_id = [self.fids[a] for a in fotoIDs]

        return (test_x1, test_x2, test_id)

