""" Siamese implementation using Tensorflow with Foto Lineup examples.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

#import system things
import tensorflow as tf
import numpy as np
import pandas as pd
import lineupsData as ld
import os
from sklearn.decomposition import PCA

#import helpers
import network
import visualize

contentMode = "cbvis"
visLayer = "FC7"

lnData = ld.LineupsData(2548, contentMode, visLayer)

print(lnData.vecCB.shape)
print(lnData.vecVis.shape)
print(lnData.pairs.shape)
print(len(lnData.fids))


# prepare data and tf.session
sess = tf.InteractiveSession()

# setup siamese network
siamese = network.SiameseDual(lnData.contentMode, lnData.dtSizeCB, lnData.dtSizeVis)

    
train_step = tf.train.AdamOptimizer(0.01).minimize(siamese.loss) #for FC7 0.002
#saver = tf.train.Saver() #save intermediate results
tf.initialize_all_variables().run()

for step in range(501):  # 50001 for FC8 and PROB
    (batch_x11, batch_x12, batch_x21, batch_x22, batch_y) = lnData.next_batch(128)

    _, loss_v = sess.run([train_step, siamese.loss], feed_dict={
        siamese.x11: batch_x11,
        siamese.x12: batch_x12,
        siamese.x21: batch_x21,
        siamese.x22: batch_x22,
        siamese.y_: batch_y})


    if np.isnan(loss_v):
        print('Model diverged with loss = NaN')
        quit()

    if step % 10 == 0:
        print('step %d: loss %.3f' % (step, loss_v))

    if step % 500 == 0 and step > 0:
        (testInput1, testInput2, testIDs) = lnData.test(lnData.allDataLength)
        embed = siamese.o1.eval({siamese.x11: testInput1, siamese.x12: testInput2})

        np.savetxt("processData/" + contentMode + visLayer + "_test_embeddings.csv", embed, delimiter=";", fmt="%.8f")
        np.savetxt("processData/" + contentMode + visLayer + "_test_ids.csv", testIDs, delimiter=";", fmt="%s")
           


# visualize result
(testInput1,testInput2, testIDs) = lnData.test(50)
embed = siamese.o1.eval({siamese.x11: testInput1,siamese.x12: testInput2})


pca = PCA(n_components=2)
pca.fit(embed)
embed2D = pca.transform(embed)
visualize.Visualize(embed2D, testIDs)